# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from quixote import get_request, redirect
from quixote.directory import Directory
from quixote.html import htmltext

def gpl():
    return htmltext("""<p>This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.</p>

    <p>This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.</p>

    <p>You should have received a copy of the GNU General Public License
    along with this program; if not, see <a href="http://www.gnu.org/licenses/"
    >http://www.gnu.org/licenses/</a>.</p>
    """)


class RootDirectory(Directory):
    menu_items = [ # still used for access control (permissions panel)
        ('forms/', N_('Forms')),
        ('workflows/', N_('Workflows')),
        ('users/', N_('Users')),
        ('roles/', N_('Roles')),
        ('categories/', N_('Categories')),
        ('logger/', N_('Logs')),
        ('bounces/', N_('Bounces')),
        ('settings/', N_('Settings')),
        ('/', N_('WCS Form Server'))]

    def _q_traverse(self, path):
        url = get_request().get_path_query()
        url = url.replace('/admin/', '/backoffice/', 1)
        return redirect(url)

    def register_page(cls, url_name, directory = None, label = None):
        if directory:
            setattr(cls, url_name, directory)
        if not label:
            label = directory.label

        if directory and not url_name.endswith('/'):
            url_name += '/'

        if not (url_name, label) in cls.menu_items:
            logger_index = cls.menu_items.index(('logger/', 'Logs'))
            cls.menu_items.insert(logger_index, (url_name, label))

    register_page = classmethod(register_page)
